if (!requireNamespace("BiocManager", quietly = TRUE))
  install.packages("BiocManager")

BiocManager::install("DESeq2")

require("DESeq2")

library("DESeq2")

library(ggplot2)

# gene_count_matrix.csv is the data file, geuvadis_phenodata.csv is the metadata file.
countData <- as.matrix(read.csv("gene_count_matrix.csv", row.names="gene_id"))
colData <- read.csv('geuvadis_phenodata.csv', sep=",", row.names="ids")

# Check all sample IDs in colData are also in CountData and match their orders
all(rownames(colData) %in% colnames(countData))
countData <- countData[, rownames(colData)]
all(rownames(colData) == colnames(countData))

# Create a DESeqDataSet from count matrix and labels
dds <- DESeqDataSetFromMatrix(countData = countData,
                              colData = colData, design = ~ genotype)


# Run the default analysis for DESeq2 and generate results table
dds <- DESeq(dds)
res <- results(dds)
head(res)
write.csv(res, "analyzed_result.csv", row.names=TRUE)

# Get the raw normalized reads from all samples
count <- counts(dds, normalized=TRUE)
write.csv(count, "count_result.csv", row.names=TRUE)

# Plot PCA analysis
# first use rlog() (regularized log transform) of the normalized counts for sample-level QC 
# as it moderates the variance across the mean, improving the clustering.
rld <- rlog(dds)
plotPCA(rld, intgroup=c("genotype"), ntop=5000)

# Plot MA-plot, adjusted p-value < 0.1 in blue
plotMA(res, ylim=c(-4,4))

# sort res by adjusted p-value from smallest to largest
res <- res[order(res$padj),]

# remove entries which contain NA values
res <- res[complete.cases(res), ]

# filter entries which have adjusted p-value < 0.01 and fold changes larger than 84 or smaller than 1/8
res <- res[res$padj<0.01,]
resup <- res[(-1*res$log2FoldChange)>3,]
resdown <- res[(1*res$log2FoldChange)>3,]

write.csv(resup, "filter_001_8fold_up.csv", row.names=TRUE)
write.csv(resdown, "filter_001_8fold_down.csv", row.names=TRUE)

# load the normalized count and then calculate z-score, then merge with the filtered list
countData1 <- as.matrix(read.csv("gene_count_matrix1.csv", row.names="gene_id"))
head(countData1)

cal_z_score <- function(x){
  (x - mean(x)) / sd(x)
}
data_subset_norm <- t(apply(countData1, 1, cal_z_score))
head(data_subset_norm)

# remove entries which contain NA values
data_subset_norm <- data_subset_norm[complete.cases(data_subset_norm), ]
head(data_subset_norm)

# merge the z-score list with filtered list
# load the filtered list
countData2 <- as.matrix(read.csv("filter_001_8fold_result.csv", row.names="gene_id"))
head(countData2)


de <- merge(countData2, data_subset_norm, by=0, all=TRUE)
head(de)
de <- de[complete.cases(de), ]
head(de)
dim(de)

# select useful columns for heatmap
library(dplyr)
df <- select(de, "Row.names", "SAMPLE1",  "SAMPLE2",  "SAMPLE3",  "SAMPLE4",  "SAMPLE5",  "SAMPLE6")
head(df)
dim(df)
write.csv(df, "for_heatmap_001_8fold.csv", row.names=TRUE)

# make heatmap
library(pheatmap)
countData3 <- read.csv("for_heatmap_001_8fold.csv", row.names="name")
head(countData3)
pheatmap(countData3)

